use crate::utils;

pub fn execute_task01(content: &str) {
    let max: i32 = solve_task01(content);

    println!("Day01 - Task01 - Biggest Calories {}", max)
}


pub fn execute_task02(content: &str) {
    let max_three: i32 = solve_task02(content);

    println!("Day01 - Task02 - Biggest Calories of top three: {}", max_three);
}

fn solve_task01(content: &str) -> i32 {
    let vector = utils::convert_to_string_slice(&content);
    let mut sum_vector = collect_all_calories(vector);
    sum_vector.sort_by_key(|a| std::cmp::Reverse(*a));

    sum_vector[0]
}

fn solve_task02(content: &str) -> i32 {
    let vector = utils::convert_to_string_slice(&content);
    let mut sum_vector = collect_all_calories(vector);
    sum_vector.sort_by_key(|a| std::cmp::Reverse(*a));

    sum_vector.iter().take(3).sum()
}

fn collect_all_calories(list: Vec<&str>) -> Vec<i32> {
    let mut calories: i32 = 0;
    let mut calories_list: Vec<i32> = vec![];

    for var in list {

        if var.is_empty() {
            calories_list.push(calories);
            calories = 0;
            continue;
        }

        calories += var.parse::<i32>().unwrap_or(0);
    }
    calories_list.push(calories);

    return calories_list;
}

#[cfg(test)]
mod test {

    const EXAMPLE01: &'static str = include_str!("input/day01/example01");


    #[test]
    fn test_first() {
        assert_eq!(super::solve_task01(EXAMPLE01), 24000);
    }

    #[test]
    fn test_second() {
        assert_eq!(super::solve_task02(EXAMPLE01), 45000);
    }
}