use std::collections::HashSet;
use crate::utils;


pub fn execute_task01(content: &str) {
    let sum = solve_task01(content);

    println!("Day03 - Task01 - Sum of Priorities: {}", sum);
}

pub fn execute_task02(content: &str) {
    let sum = solve_task02(content);

    println!("Day03 - Task02 - Sum of Groups of Priorities: {}", sum);
}

fn solve_task01(content: &str) -> i32 {
    let vector = utils::convert_to_string_slice(&content);

    vector.into_iter()
        .map(|x| [&x[..x.len()/2], &x[x.len()/2..]] // create package to compare
            .into_iter()
            .map(|x| x.chars().collect::<HashSet<_>>()) // remove duplicates per pack
            .reduce(|acc, x| &acc & &x).unwrap()) // filter for only duplicates in all packages
        .map(|x| *x.iter().next().unwrap()) // take first item in every set
        .map(|c| convert_char_to_prio(c) as i32) // convert character to number
        .sum()
}

fn solve_task02(content: &str) -> i32 {
    let vector = utils::convert_to_string_slice(&content);

    vector.chunks(3).into_iter()
        .map(|content| [content[0], content[1], content[2]].into_iter()
            .map(|x| x.chars().collect::<HashSet<_>>())
            .reduce(|acc, x| &acc & &x).unwrap())
        .map(|x| *x.iter().next().unwrap())
        .map(|c| convert_char_to_prio(c) as i32)
        .sum()
}

pub fn convert_char_to_prio(c: char) -> u8 {
    let c = c as u8;
    match c {
        b'a'..=b'z' => c - b'a' + 1,
        b'A'..=b'Z' => c - b'A' + 27,
        _ => unreachable!(),
    }
}

#[cfg(test)]
mod test {
    const EXAMPLE01: &'static str = include_str!("input/day03/example01");


    #[test]
    fn test_first() {
        assert_eq!(super::solve_task01(EXAMPLE01), 157);
    }

    #[test]
    fn test_second() {
        assert_eq!(super::solve_task02(EXAMPLE01), 70);
    }
}